use Test::More;
BEGIN {
	use Hades;
	Hades->run({
		eval => 'Kosmos { penthos :d(2) curae :r nosoi :default(2) geras { if ($_[0]->penthos == $_[0]->nosoi) { return $_[0]->curae; } } }',
		lib => 't/lib'
	});
	use lib 't/lib';
}
use Kosmos;
my $okay = Kosmos->new({
	curae => 5
});
is($okay->penthos, 2);
is($okay->nosoi, 2);
is($okay->curae, 5);
is($okay->geras, 5);
done_testing;
